import path
import zipfile
import StringIO
import sys
import os

class EasyZip(object):
    def __init__(self,dst,flag='w'):
        dst = path.path(dst)
        self.mkdirs(dst)
        self.z = zipfile.ZipFile(dst,flag)
        self.dirs = []
    
    def __del__(self): # close the archive
        self.close()
        
    def extract(self,dst='./'): # extract the files of the archive
        dst = path.path(dst)
        for f in self.files():
            c = dst.joinpath(f)
            self.extractFile(f,c)
    
    def extractFile(self,src,dst='./'): # extract a single file
        dst = dst.replace('\\','/')
        if dst[-1] == '/':
            dst = path.path(dst).joinpath(path.path(src).basename())
        else:
            dst = path.path(dst)
        self.mkdirs(dst)
        dst.write_bytes(self.__read(src))

    def mkdirs(self,dst):
        old = 0
        new = dst.dirname()
        dirs = []
        while old != new:
            dirs.append(new)
            old = new
            new = new.dirname()
        dirs.reverse()
        for d in dirs:
            if not d.isdir():
                d.mkdir()
            
    def __rshift__(self,dst):
        self.extract(dst)
    
    def addFile(self,src,dst='/',compress_type=None): # add a file
        dst = dst.replace('\\','/')
        src = path.path(src)
        if dst[-1] == '/': # destination is a folder
            self._addDir(dst)
            dst = path.path(dst).joinpath(src.basename())
            dst = dst + src.basename()
        self._addFile(src,dst,compress_type)
        
    def __iadd__(self,info):
        src = info
        dst = '/'
        compress_type = None
        if type(info) in (tuple,list):
            if len(info) >= 1:
                src = info[0]
            if len(info) >= 2:
                dst = info[1]
            if len(info) == 3:
                compress_type = info[2]
        psrc = path.path(src)
        if psrc.isfile():
            self.addFile(src,dst,compress_type)
        elif psrc.isdir():
            self.addDir(src,dst,compress_type)
        else:
            self.addFileContent(src,dst,compress_type)
        return self
    
    def __lshift__(self,info):
        self.__iadd__(info)
        
    def addFileContent(self,content,dst='/',compress_type=None):
        content = StringIO.StringIO(content)
        dst = dst.replace('\\','/')
        src = path.path(src)
        if dst[-1] == '/': # destination is a folder
            self._addDir(dst)
            dst = path.path(dst).joinpath(src.basename())
            dst = dst + src.basename()
        self._addFile(content,dst,compress_type)        
    
    def addDir(self,src_root,dst_root='/',pattern='',compress_type=None):
        # add a directory to the archive
        src = path.path(src_root)
        for f in src.walkfiles(pattern):
            self.addFile(f,'%s/%s' % (dst_root,src.relpathto(f)),compress_type)
    
    def files(self): # return a list of files
        return filter(lambda x: x[-1] != '/',self.z.namelist())
        
    def _addFile(self, filename, arcname, compress_type): # Put the bytes from filename into the archive under the name arcname.
        self.z.write(filename,arcname,compress_type)
    
    def _addDir(self,dir):
        dir = path.path(dir)
        if dir in self.dirs:
            return
        while dir != dir.dirname():
            if dir in self.dirs: continue    
            zfi = zipfile.ZipInfo('%s/'%dir)
            self.__writestr(zfi, '')
            self.dirs.append(dir)
            dir = dir.dirname()
    
    def close(self): # close the file
        self.z.close()
        
    def __writestr(self, zinfo_or_arcname, bytes):
        self.z.writestr(zinfo_or_arcname,bytes)
    
    def __read(self,src):
        return self.z.read(src)

args = sys.argv[1:]
gamedir = args.pop(0)
server_command = ' '.join(args)

gamedir = path.path(gamedir)
temp = gamedir.joinpath('addons/eventscripts/updater/temp.zip')
vdffile = gamedir.joinpath('addons/mattie_eventscripts.vdf')

vdftemplate = '''"Plugin"
{
        "file"  "../$$$game$$$/addons/mattie_eventscripts"
}'''

def vdf():
    game = gamedir.namebase
    if not game:
        game = gamedir.dirname().namebase
    vdffile.write_text(vdftemplate.replace('$$$game$$$',game))

def updater():
    z = EasyZip(temp,'r')
    print 'Extracting files...'
    z.extract(gamedir)
    print 'Extracting files... Done'
    print 'Verifying VDF... '
    vdf()
    print 'Cleaning up...'
    temp.remove()
    os.system(server_command)

updater()
